From f8c1913bd9fb332bfa3ff722f6870216d5194c6f Mon Sep 17 00:00:00 2001
From: James Moger <james.moger@gitblit.com>
Date: Sun, 16 Mar 2014 16:26:19 -0400
Subject: [PATCH] Merged #32 "Fix incorrect ticket branch ref in ticket page changed paths list"
---
src/main/java/com/gitblit/tickets/BranchTicketService.java | 109 +++++++++++++++++++++++++++++++++++++++++-------------
1 files changed, 83 insertions(+), 26 deletions(-)
diff --git a/src/main/java/com/gitblit/tickets/BranchTicketService.java b/src/main/java/com/gitblit/tickets/BranchTicketService.java
index 60dca27..f23a325 100644
--- a/src/main/java/com/gitblit/tickets/BranchTicketService.java
+++ b/src/main/java/com/gitblit/tickets/BranchTicketService.java
@@ -27,9 +27,8 @@
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
-
-import javax.inject.Inject;
import org.eclipse.jgit.api.errors.ConcurrentRefUpdateException;
import org.eclipse.jgit.api.errors.JGitInternalException;
@@ -44,21 +43,26 @@
import org.eclipse.jgit.lib.ObjectId;
import org.eclipse.jgit.lib.ObjectInserter;
import org.eclipse.jgit.lib.PersonIdent;
+import org.eclipse.jgit.lib.Ref;
+import org.eclipse.jgit.lib.RefRename;
import org.eclipse.jgit.lib.RefUpdate;
import org.eclipse.jgit.lib.RefUpdate.Result;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.revwalk.RevCommit;
import org.eclipse.jgit.revwalk.RevTree;
import org.eclipse.jgit.revwalk.RevWalk;
+import org.eclipse.jgit.transport.ReceiveCommand;
import org.eclipse.jgit.treewalk.CanonicalTreeParser;
import org.eclipse.jgit.treewalk.TreeWalk;
import com.gitblit.Constants;
+import com.gitblit.git.ReceiveCommandEvent;
import com.gitblit.manager.INotificationManager;
import com.gitblit.manager.IRepositoryManager;
import com.gitblit.manager.IRuntimeManager;
import com.gitblit.manager.IUserManager;
import com.gitblit.models.PathModel;
+import com.gitblit.models.PathModel.PathChangeModel;
import com.gitblit.models.RefModel;
import com.gitblit.models.RepositoryModel;
import com.gitblit.models.TicketModel;
@@ -78,19 +82,7 @@
*/
public class BranchTicketService extends ITicketService implements RefsChangedListener {
- /**
- * The event fired by other classes to allow this service to index tickets.
- */
- public static class TicketsBranchUpdated extends RefsChangedEvent {
-
- public final RepositoryModel model;
-
- public TicketsBranchUpdated(RepositoryModel model) {
- this.model = model;
- }
- }
-
- public static final String BRANCH = "refs/gitblit/tickets";
+ public static final String BRANCH = "refs/meta/gitblit/tickets";
private static final String JOURNAL = "journal.json";
@@ -98,7 +90,6 @@
private final Map<String, AtomicLong> lastAssignedId;
- @Inject
public BranchTicketService(
IRuntimeManager runtimeManager,
INotificationManager notificationManager,
@@ -111,7 +102,7 @@
repositoryManager);
lastAssignedId = new ConcurrentHashMap<String, AtomicLong>();
-
+
// register the branch ticket service for repository ref changes
Repository.getGlobalListenerList().addRefsChangedListener(this);
}
@@ -138,39 +129,105 @@
}
/**
- * Listen for refs changed events and reindex that repository.
+ * Listen for tickets branch changes and (re)index tickets, as appropriate
*/
@Override
- public void onRefsChanged(RefsChangedEvent event) {
- if (!(event instanceof TicketsBranchUpdated)) {
+ public synchronized void onRefsChanged(RefsChangedEvent event) {
+ if (!(event instanceof ReceiveCommandEvent)) {
return;
}
- RepositoryModel repository = ((TicketsBranchUpdated) event).model;
+
+ ReceiveCommandEvent branchUpdate = (ReceiveCommandEvent) event;
+ RepositoryModel repository = branchUpdate.model;
+ ReceiveCommand cmd = branchUpdate.cmd;
try {
- reindex(repository);
+ switch (cmd.getType()) {
+ case CREATE:
+ case UPDATE_NONFASTFORWARD:
+ // reindex everything
+ reindex(repository);
+ break;
+ case UPDATE:
+ // incrementally index ticket updates
+ resetCaches(repository);
+ long start = System.nanoTime();
+ log.info("incrementally indexing {} ticket branch due to received ref update", repository.name);
+ Repository db = repositoryManager.getRepository(repository.name);
+ try {
+ Set<Long> ids = new HashSet<Long>();
+ List<PathChangeModel> paths = JGitUtils.getFilesInRange(db,
+ cmd.getOldId().getName(), cmd.getNewId().getName());
+ for (PathChangeModel path : paths) {
+ String name = path.name.substring(path.name.lastIndexOf('/') + 1);
+ if (!JOURNAL.equals(name)) {
+ continue;
+ }
+ String tid = path.path.split("/")[2];
+ long ticketId = Long.parseLong(tid);
+ if (!ids.contains(ticketId)) {
+ ids.add(ticketId);
+ TicketModel ticket = getTicket(repository, ticketId);
+ log.info(MessageFormat.format("indexing ticket #{0,number,0}: {1}",
+ ticketId, ticket.title));
+ indexer.index(ticket);
+ }
+ }
+ long end = System.nanoTime();
+ log.info("incremental indexing of {0} ticket(s) completed in {1} msecs",
+ ids.size(), TimeUnit.NANOSECONDS.toMillis(end - start));
+ } finally {
+ db.close();
+ }
+ break;
+ default:
+ log.warn("Unexpected receive type {} in BranchTicketService.onRefsChanged" + cmd.getType());
+ break;
+ }
} catch (Exception e) {
log.error("failed to reindex " + repository.name, e);
}
}
/**
* Returns a RefModel for the refs/gitblit/tickets branch in the repository.
* If the branch can not be found, null is returned.
*
* @return a refmodel for the gitblit tickets branch or null
*/
private RefModel getTicketsBranch(Repository db) {
- List<RefModel> refs = JGitUtils.getRefs(db, Constants.R_GITBLIT);
+ List<RefModel> refs = JGitUtils.getRefs(db, "refs/");
+ Ref oldRef = null;
for (RefModel ref : refs) {
if (ref.reference.getName().equals(BRANCH)) {
return ref;
+ } else if (ref.reference.getName().equals("refs/gitblit/tickets")) {
+ oldRef = ref.reference;
+ }
+ }
+ if (oldRef != null) {
+ // rename old ref to refs/meta/gitblit/tickets
+ RefRename cmd;
+ try {
+ cmd = db.renameRef(oldRef.getName(), BRANCH);
+ cmd.setRefLogIdent(new PersonIdent("Gitblit", "gitblit@localhost"));
+ cmd.setRefLogMessage("renamed " + oldRef.getName() + " => " + BRANCH);
+ Result res = cmd.rename();
+ switch (res) {
+ case RENAMED:
+ log.info(db.getDirectory() + " " + cmd.getRefLogMessage());
+ return getTicketsBranch(db);
+ default:
+ log.error("failed to rename " + oldRef.getName() + " => " + BRANCH + " (" + res.name() + ")");
+ }
+ } catch (IOException e) {
+ log.error("failed to rename tickets branch", e);
}
}
return null;
}
/**
* Creates the refs/gitblit/tickets branch.
* @param db
*/
private void createTicketsBranch(Repository db) {
@@ -183,7 +240,7 @@
* folder with the remaining characters as a subfolder within that folder.
*
* @param ticketId
* @return the root path of the ticket content on the refs/gitblit/tickets branch
*/
private String toTicketPath(long ticketId) {
StringBuilder sb = new StringBuilder();
@@ -138,39 +129,105 @@
}
/**
- * Returns a RefModel for the refs/gitblit/tickets branch in the repository.
+ * Returns a RefModel for the refs/meta/gitblit/tickets branch in the repository.
* If the branch can not be found, null is returned.
*
@@ -138,39 +129,105 @@
}
/**
- * Creates the refs/gitblit/tickets branch.
+ * Creates the refs/meta/gitblit/tickets branch.
* @param db
*/
@@ -183,7 +240,7 @@
*
* @param ticketId
- * @return the root path of the ticket content on the refs/gitblit/tickets branch
+ * @return the root path of the ticket content on the refs/meta/gitblit/tickets branch
*/
private String toTicketPath(long ticketId) {
--
Gitblit v1.9.1